from propka.run import single
import sys

if len(sys.argv) < 2:
    print("Usage: python pka.py <pdb_file>")
    sys.exit(1)

pdb_file = sys.argv[1]
ph = 7.4
if len(sys.argv) > 2:
    try:
        ph = float(sys.argv[2])
    except ValueError:
        print("Invalid pH value. Using default pH 7.4.")


mol = single(
    pdb_file,
    optargs=[f"--pH={ph}", '-q'],
)


#!/usr/bin/env python3

import subprocess
import sys
import re
import os
import shutil
import argparse
from pathlib import Path


# Package root: .../site-packages/aaby
PACKAGE_ROOT = Path(__file__).resolve().parents[1]
RESOURCES_DIR = PACKAGE_ROOT / "resources"
MDPS_DIR = RESOURCES_DIR / "mdps"



def gen_rest(path, itp, mol2=None):
    with open(path + itp) as f:
        lines = f.readlines()

    if 'protein' in itp:
        atoms = False
        res_BB = []
        res_SC = []
        for line in lines:
            if atoms and line[0] != ';':
                if line == '\n':
                    break
                l = line.split()
                if l[4] in ['N','CA', 'C', 'O']:
                    # print(line, end='')
                    res_BB.append([l[0], l[4]])
                elif l[4][0] != 'H':
                    # print(line, end='')
                    res_SC.append([l[0], l[4]])

            if '[ atoms ]' in line:
                atoms = True
        restraint = ''
        restraint += '#ifdef POSRES\n'
        restraint +='[ position_restraints ]\n'
        for i in res_BB:
            restraint += f'{i[0]:>5}     1     POSRES_FC_BB    POSRES_FC_BB    POSRES_FC_BB\n'
        for i in res_SC:
            restraint += f'{i[0]:>5}     1     POSRES_FC_SC    POSRES_FC_SC    POSRES_FC_SC\n'
        restraint += '#endif\n'
        with open(path + itp, 'a') as f:
            f.write(restraint)

    
    elif itp.split('.')[0] in ['CHL', 'DLPC', 'DMPC', 'DPPC', 'DSPC', 'DOPC', 'POPC', 'POPE', 'DLPG', 'DMPG', 'DPPG', 'DSPG', 'DOPG', 'POPG', 'DOPS', 'POPS', 'POPA', 'DAPC', 'SDPC', 'PSM', 'SSM']:
        atoms = False
        res = []
        for line in lines:
            if atoms and line[0] != ';':
                if line == '\n':
                    break
                l = line.split()
                if l[4] in ['P31', 'O1']:
                    # print(l)
                    res.append([l[0], l[4]])
            if '[ atoms ]' in line:
                atoms = True
                
        restraint = ''
        restraint += '#ifdef POSRES\n'
        restraint += '[ position_restraints ]\n'
        for i in res:
            restraint += f'{i[0]}     1     0.0             0.0            POSRES_FC_LIPID\n'
        restraint += '#endif\n'
        
        
        with open(path + itp, 'a') as f:
            f.write(restraint)

    elif mol2 in itp:
        atoms = False
        res_BB = []
        res_SC = []
        for line in lines:
            if atoms and line[0] != ';':
                if line == '\n':
                    break
                l = line.split()
                if l[4][0] != 'H':
                    # print(line, end='')
                    res_BB.append([l[0], l[4]])

            if '[ atoms ]' in line:
                atoms = True
        restraint = ''
        restraint += '#ifdef POSRES\n'
        restraint +='[ position_restraints ]\n'
        for i in res_BB:
            restraint += f'{i[0]:>5}     1     POSRES_FC_BB    POSRES_FC_BB    POSRES_FC_BB\n'
        restraint += '#endif\n'
        with open(path + itp, 'a') as f:
            f.write(restraint)

def main(mol2=None):
    path = 'toppar/'
    itps = os.listdir(path)
    for itp in itps:
        gen_rest(path, itp, mol2)
    print(f"[DONE] Restraints added to itp files")

if __name__ == "__main__":
    main(sys.argv[1])

    


import argparse

def parse_args():
    parser = argparse.ArgumentParser(description="AABY: End-to-end AMBER system builder from PDB")
    parser.add_argument('-f', '--input', required=True, help='Input PDB file')
    parser.add_argument('--chains', default="A,B,C,D", help='Comma-separated list of protein chains for ACE/NME')
    parser.add_argument('--ssbond', action='store_true', help='Renumber SSBOND.txt, convert CYS to CYX, insert ssbonds into tleap.in')
    group = parser.add_mutually_exclusive_group()
    group.add_argument('--ph', type=float, help='pH for propka (protonation by predicted pKa)')
    group.add_argument('--protlist', type=str, help='Residue list file for direct protonation changes')
    parser.add_argument('--coby-yaml', type=str, help='YAML file specifying COBY arguments (use {protein_pdb} as placeholder)')
    parser.add_argument('--coby-args', nargs=argparse.REMAINDER, help='Arguments passed directly to COBY after this flag (use {protein_pdb} as placeholder)')
    return parser.parse_args()



import shutil
from pathlib import Path
import yaml
from .utils import run, autodetect_amber_files, substitute_protein_pdb, make_coby_ter_pdb, insert_ssbonds_into_tleap

def run_pipeline(args):
    pdb = Path(args.input)
    base = pdb.stem

    # Example: Copy tleap.in from package data to working dir
    try:
        from importlib.resources import files
        tleap_template = files("aaby.data").joinpath("tleap.in")
        shutil.copy(tleap_template, "tleap.in")
    except Exception:
        import pkg_resources
        tleap_template = pkg_resources.resource_filename("aaby", "data/tleap.in")
        shutil.copy(tleap_template, "tleap.in")

    # ...continue the workflow as in your previous script...
    # Use run(), autodetect_amber_files(), etc as imported from utils.py
    # For brevity, you can paste your refactored logic here, calling functions for each step.
    print("Pipeline would run here. See workflow.py for extension.")

    # This is a skeleton; you can now fill in each workflow step as a function.



import sys
import os
import re
from parmed import load_file
import shutil

def parmed_convert(prmtop_file, inpcrd_file, top_file, gro_file):
    amber = load_file(prmtop_file, inpcrd_file)
    amber.save(gro_file, format="gro", overwrite=True)
    amber.save(top_file, format="gromacs", overwrite=True)
    print(f"Wrote {gro_file} and {top_file}")

def split_top_to_itps(top_file, toppar_dir="toppar", output_top='topol.top'):
    with open(top_file, 'r') as f:
        lines = f.readlines()

    # Gather includes and preamble up to first [ moleculetype ]
    preamble = []
    i = 0
    while i < len(lines):
        if re.match(r'^\s*\[\s*moleculetype\s*\]', lines[i], re.IGNORECASE):
            break
        preamble.append(lines[i])
        i += 1

    # Extract each moleculetype block into its own .itp
    itp_blocks = []
    itp_names = []
    while i < len(lines):
        if re.match(r'^\s*\[\s*moleculetype\s*\]', lines[i], re.IGNORECASE):
            block = []
            while i < len(lines):
                if (re.match(r'^\s*\[\s*moleculetype\s*\]', lines[i], re.IGNORECASE) and block) or \
                   (re.match(r'^\s*\[\s*system\s*\]', lines[i], re.IGNORECASE)):
                    break
                block.append(lines[i])
                i += 1
            # Get moleculetype name for filename and [ molecules ] block
            for j, line in enumerate(block):
                if j == 0:
                    continue
                if not line.strip().startswith(';') and line.strip():
                    mtname = line.split()[0]
                    itp_names.append(mtname)
                    itp_blocks.append(block)
                    break
        else:
            i += 1

    # Find [ molecules ] section to extract molecule counts
    mol_section = []
    found_mol = False
    for idx, line in enumerate(lines):
        if re.match(r'^\s*\[\s*molecules\s*\]', line, re.IGNORECASE):
            found_mol = True
            continue
        if found_mol:
            if line.strip().startswith('['):
                break
            if line.strip() and not line.strip().startswith(';'):
                mol_section.append(line)

    # Write ITP files to toppar directory
    os.makedirs(toppar_dir, exist_ok=True)
    for name, block in zip(itp_names, itp_blocks):
        fname = os.path.join(toppar_dir, f"{name}.itp")
        with open(fname, "w") as fout:
            fout.writelines(block)
        print(f"Wrote {fname}")

    # Write new topol.top with #include "toppar/NAME.itp"
    with open(output_top, "w") as f:
        f.writelines(preamble)
        for name in itp_names:
            f.write(f'#include "{toppar_dir}/{name}.itp"\n')
        f.write("\n[ system ]\n; Name\nGeneric title\n\n")
        f.write("[ molecules ]\n; Compound       #mols\n")
        for line in mol_section:
            f.write(line)
    print(f"Wrote {output_top}")

def split_ff_from_topol(topol_file, ff_file="ff.itp", new_topol="topol.top"):
    with open(topol_file, "r") as f:
        lines = f.readlines()

    ff_block = []
    rest_block = []
    found_first_include = False

    for idx, line in enumerate(lines):
        if not found_first_include and line.strip().startswith("#include"):
            found_first_include = True
            rest_block = lines[idx:]  # include all from this include onward
            break
        if not found_first_include:
            ff_block.append(line)

    # Write ff.itp
    with open(ff_file, "w") as fff:
        fff.writelines(ff_block)
    print(f"Wrote {ff_file}")

    # Write new topol.top (include ff.itp at top, then the rest)
    with open(new_topol, "w") as ftop:
        ftop.write(f'#include "{ff_file}"\n')
        for line in rest_block:
            ftop.write(line)
    print(f"Wrote {new_topol}")

def main(prmtop_file, inpcrd_file, top_file, gro_file, topol):
    # top_file = "system.top"
    # gro_file = "system.gro"
    parmed_convert(prmtop_file, inpcrd_file, top_file, gro_file)
    split_top_to_itps(top_file, toppar_dir="toppar", output_top='topol.top.temp')
    split_ff_from_topol('topol.top.temp', ff_file="ff.itp", new_topol=f"{topol}.top")
    os.remove("topol.top.temp")

if __name__ == "__main__":
    if len(sys.argv) != 6:
        print("Usage: python convert_and_split_full.py file.prmtop file.inpcrd, top_file.top, gro_file.gro, topol")
        sys.exit(1)
    main(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5])



from pathlib import Path

pdb_file = Path("COBY_TER.pdb")
output_file = Path("COBY_TER_lipid21.pdb")

# Define POPC block structure

#ranges_per_lipid = {'POPC': (1, 46, "PA"), (47, 84, "PC"), (85, 134, "OL")], 
#'POPE': [(1, 46, "PA"), (47, 75, "PC"), (76, 125, "OL")]}

ranges_per_lipid = {
"DAPC": [(1, 50, "AR"), (51, 88, "PC"), (89, 138, "AR")],
"DLPC": [(1, 34, "LAL"), (35, 72, "PC"), (73, 106, "LAL")],
"DLPG": [(1, 34, "LAL"), (35, 65, "PGR"), (66, 99, "LAL")],
"DMPC": [(1, 40, "MY"), (41, 78, "PC"), (79, 118, "MY")],
"DMPG": [(1, 40, "MY"), (41, 71, "PGR"), (72, 111, "MY")],
"DOPC": [(1, 50, "OL"), (51, 88, "PC"), (89, 138, "OL")],
"DOPG": [(1, 50, "OL"), (51, 81, "PGR"), (82, 131, "OL")],
"DOPS": [(1, 50, "OL"), (51, 81, "PS"), (82, 131, "OL")],
"DPPC": [(1, 46, "PA"), (47, 84, "PC"), (85, 130, "PA")],
"DPPG": [(1, 46, "PA"), (47, 77, "PGR"), (78, 123, "PA")],
"DSPC": [(1, 52, "ST"), (53, 90, "PC"), (91, 142, "ST")],
"DSPG": [(1, 52, "ST"), (53, 83, "PGR"), (84, 135, "ST")],
"POPA": [(1, 46, "PA"), (47, 66, "PH-"), (67, 116, "OL")],
"POPC": [(1, 46, "PA"), (47, 84, "PC"), (85, 134, "OL")],
"POPE": [(1, 46, "PA"), (47, 75, "PE"), (76, 125, "OL")],
"POPG": [(1, 46, "PA"), (47, 77, "PGR"), (78, 127, "OL")],
"POPS": [(1, 46, "PA"), (47, 77, "PS"), (78, 127, "OL")],
"PSM": [(1, 46, "PA"), (47, 83, "SPM"), (84, 127, "SA")],
"SDPC": [(1, 52, "ST"), (53, 90, "PC"), (91, 142, "DHA")],
"SSM": [(1, 52, "ST"), (53, 89, "SPM"), (90, 133, "SA")]}

#atoms_per_lipid = {'POPC': 134, 'POPE': 125}

atoms_per_lipid = {'DAPC': 138,
'DLPC': 106,
'DLPG': 99,
'DMPC': 118,
'DMPG': 111,
'DOPC': 138,
'DOPG': 131,
'DOPS': 131,
'DPPC': 130,
'DPPG': 123,
'DSPC': 142,
'DSPG': 135,
'POPA': 116,
'POPC': 134,
'POPE': 125,
'POPG': 127,
'POPS': 127,
'PSM': 127,
'SDPC': 142,
'SSM': 133}

lipids = ['DAPC', 'DLPC', 'DLPG', 'DMPC', 'DMPG', 'DOPC', 'DOPG', 'DOPS', 'DPPC', 'DPPG', 'DSPC', 'DSPG', 'POPA', 'POPC', 'POPE', 'POPG', 'POPS', 'PSM', 'SDPC', 'SSM']

def get_resname(index_in_block, lipid):
    for start, end, resname in ranges_per_lipid[lipid]:       
        if start <= index_in_block <= end:
            return resname
    raise ValueError(f"Atom index {index_in_block} not in any range")

def split_popc_and_add_ter(pdb_path, output_path):
    lines_out = []
    popc_atom_counter = 0
    current_resid = None

    for line in open(pdb_path):
        if line.startswith(("ATOM", "HETATM")) and line[17:21].strip() in lipids:
            lipid = line[17:21].strip()
            popc_atom_counter += 1
            index_in_block = (popc_atom_counter - 1) % atoms_per_lipid[lipid] + 1

            # Save original residue number
            resid_str = line[22:26]
            if index_in_block == 1:
                current_resid = resid_str

            new_resname = get_resname(index_in_block, lipid)

            # Apply new resname, keep old resid
            line = line[:17] + new_resname.ljust(4) + line[21:22] + current_resid + line[26:]

            # Insert TER after every 134 atoms (i.e. at end of each POPC block)
            if index_in_block ==  atoms_per_lipid[lipid]:
                lines_out.append(line)
                lines_out.append(f"TER\n")
                popc_atom_counter = 0
                continue

        lines_out.append(line)

    with open(output_path, "w") as f:
        f.writelines(lines_out)

split_popc_and_add_ter(pdb_file, output_file)
print(f"✅ Lipids split into lipid21 naming (residues preserved, TERs inserted) → {output_file}")



from pathlib import Path
import string

input_file = Path("COBY_TER_lipid21.pdb")
output_file = input_file.with_name(input_file.stem + "_chains.pdb")

protein_residues = {
    "ALA", "ARG", "ASN", "ASP", "CYS", "GLN", "GLU", "GLY",
    "HIS", "ILE", "LEU", "LYS", "MET", "PHE", "PRO", "SER",
    "THR", "TRP", "TYR", "VAL", "HIE", "HID", "HIP", "CYX",
    "LYN", "GLH", "ASH", "ACE", "NME","Na", "Cl","K", "Ca", "Mg", "Zn","NA", "CL","K", "CA", "MG", "ZN"
}

lipid_residues = ['SA', 'PGR', 'AR', 'LAL', 'ST', 'DHA', 'PE', 'MY', 'OL', 'PH-', 'PS', 'PC', 'SPM', 'PA','CHL']


chain_labels = list(string.ascii_uppercase)
current_chain = 0
in_protein_block = False

with open(input_file) as inp, open(output_file, "w") as out:
    for line in inp:
        if line.startswith(("ATOM", "HETATM")):
            resname = line[17:20].strip()
            if resname not in lipid_residues:
                line = line[:21] + chain_labels[current_chain] + line[22:]
                in_protein_block = True
            else:
                line = line[:21] + "X" + line[22:]
                in_protein_block = False
            out.write(line)
        elif line.startswith("TER"):
            # Only modify if TER line is long enough (has chain info)
            if len(line.rstrip()) == 3:  # 'TER\n' or 'TER\r\n'
                out.write(line)
            elif len(line) >= 22:
                if in_protein_block:
                    line = line[:21] + chain_labels[current_chain] + line[22:]
                    current_chain += 1
                else:
                    line = line[:21] + "X" + line[22:]
                in_protein_block = False
                out.write(line)
            else:
                out.write(line)
        else:
            out.write(line)

print(f"Wrote: {output_file}")



import subprocess
from pathlib import Path

def run(cmd, shell=True, check=True):
    if isinstance(cmd, list):
        print(f"Running: {' '.join(str(x) for x in cmd)}")
    else:
        print(f"Running: {cmd}")
    subprocess.run(cmd, shell=shell, check=check)

def autodetect_amber_files():
    prmtop_candidates = sorted(Path('.').glob('*.prmtop'), key=lambda f: f.stat().st_mtime, reverse=True)
    inpcrd_candidates = sorted(Path('.').glob('*.inpcrd'), key=lambda f: f.stat().st_mtime, reverse=True)
    if not prmtop_candidates or not inpcrd_candidates:
        raise RuntimeError("tleap failed to produce .prmtop or .inpcrd")
    prmtop = prmtop_candidates[0]
    inpcrd = inpcrd_candidates[0]
    print(f"Using prmtop: {prmtop}")
    print(f"Using inpcrd: {inpcrd}")
    return prmtop, inpcrd

def substitute_protein_pdb(args_list, protein_pdb):
    return [str(x).replace("{protein_pdb}", str(protein_pdb)) for x in args_list]

def make_coby_ter_pdb(coby_pdb, ter_pdb, coby_ter_pdb):
    # (Insert logic here, as previously shown)
    pass

def insert_ssbonds_into_tleap(tleap_in_path, ssbond_path):
    # (Insert logic here, as previously shown)
    pass



import sys
import argparse

alt = {
    'ASP': 'ASH',
    'GLU': 'GLH',
    'LYS': 'LYN',
    'HIS': 'HIP'
}

def parse_pka(pka_file, ph):
    acid = {}
    base = {}
    his = {}
    with open(pka_file) as f:
        lines = f.readlines()
    c1 = False
    for line in lines:
        if c1:
            if line[0] == '-':
                break
            l = line.split()
            if l[0] in ['ASP', 'GLU'] and float(l[3]) - ph > 0:
                acid[f'{l[0]}-{l[1]}-{l[2]}'] = float(l[3]) - ph
            if l[0] in ['LYS'] and float(l[3]) - ph < 0:
                base[f'{l[0]}-{l[1]}-{l[2]}'] = float(l[3]) - ph
            if l[0] == 'HIS' and float(l[3]) - ph > 0:
                his[f'{l[0]}-{l[1]}-{l[2]}'] = float(l[3]) - ph
        if line == '       Group      pKa  model-pKa   ligand atom-type\n':
            c1 = True
    selected = []
    for k in list(acid.keys()) + list(base.keys()) + list(his.keys()):
        l = k.split('-')
        resname, resid, chain = l[0], l[1], l[2]
        selected.append((resname, chain, resid))
    return selected

def parse_list(listfile):
    selected = []
    with open(listfile) as f:
        for line in f:
            if not line.strip() or line.strip().startswith("#"):
                continue
            parts = line.split()
            if len(parts) != 3:
                print(f"Warning: skipping malformed line: {line.strip()}", file=sys.stderr)
                continue
            resname, chain, resid = parts
            selected.append((resname, chain, resid))
    return selected

def write_sed_cmds(selected, outfile):
    with open(outfile, "w") as f:
        f.write("#!/bin/bash\n\n")
        for resname, chain, resid in selected:
            if resname not in alt:
                print(f"Warning: {resname} not in supported list, skipping", file=sys.stderr)
                continue
            f.write(f'sed -i "s/{resname} {chain}{int(resid):>4}/{alt[resname]} {chain}{int(resid):>4}/g" $1\n')

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("pka_or_list", help="pKa file or residue list file")
    # parser.add_argument("pdb_file", help="Target PDB file (for sed script)")
    parser.add_argument("--ph", type=float, default=7.4, help="pH value (default: 7.4)")
    parser.add_argument("--list", action="store_true", help="Treat input as residue list, not pKa file")
    parser.add_argument("-o", "--outfile", default="mutate_residues.sh", help="Output bash file (default: mutate_residues.sh)")
    args = parser.parse_args()
    if args.list:
        selected = parse_list(args.pka_or_list)
    else:
        selected = parse_pka(args.pka_or_list, args.ph)
    write_sed_cmds(selected, args.outfile)
    print(f"Wrote sed commands to {args.outfile}")

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import subprocess
import sys
import re
import os
import shutil
import argparse
from pathlib import Path


# Package root: .../site-packages/aaby
PACKAGE_ROOT = Path(__file__).resolve().parents[1]
RESOURCES_DIR = PACKAGE_ROOT / "resources"
MDPS_DIR = RESOURCES_DIR / "mdps"



# ---- USER SETTINGS ----
input_gro = "system.gro"
input_top = "topol.top"
input_top = "topol_nowater.top"

# -----------------------

intermediate_gro = "noions.gro"
intermediate_top = "noions.top"
resized_gro = "resized.gro"
minimized_gro = "membrane.gro"
resolvated_gro = "resolvated.gro"
resolvated_top = "resolvated.top"
cleaned_gro = "system_solv.gro"
cleaned_top = "topol_updated.top"
gro_wat = "system_solv_wat.gro"
top_wat = "topol_updated_wat.top"
ndx = "solv.ndx"
tpr = "ions.tpr"

REMOVE_RESNAMES = ["WAT", "SOL", "Na", "Cl","K", "Ca", "Mg", "Zn"]

def run(cmd, return_output=False):
    print("[RUN]", cmd)
    if return_output:
        result = subprocess.run(cmd, shell=True, capture_output=True, text=True)
        return result.stdout
    else:
        subprocess.run(cmd, shell=True, check=True)

def remove_ions_and_water_gro(input_gro, output_gro, remove_resnames=None):
    if remove_resnames is None:
        remove_resnames = ["WAT", "SOL", "Na", "Cl","K", "Ca", "Mg", "Zn"]
    remove_resnames_set = set([x.upper() for x in remove_resnames])
    with open(input_gro) as f:
        lines = f.readlines()
    new_lines = [lines[0], lines[1]]
    natoms = 0
    for l in lines[2:-1]:
        if len(l) < 10:
            continue
        resname = l[5:10].strip()
        if resname not in remove_resnames:
            new_lines.append(l)
            natoms += 1
    new_lines.append(lines[-1])
    new_lines[1] = f"{natoms}\n"
    with open(output_gro, "w") as f:
        f.writelines(new_lines)

def resize_box(input_gro, output_gro):
    z_length = float(run(f"tail -1 {input_gro} | awk '{{print $3}}'", return_output=True).strip())
    run(f'gmx editconf -f {input_gro} -d -.15 -o {output_gro}')
    new_z_length = float(run(f"tail -1 {output_gro} | awk '{{print $3}}'", return_output=True).strip())
    last_line_nr = int(run(f"wc -l {output_gro} | awk '{{print $1}}'", return_output=True).strip())
    run(f'sed -i "{last_line_nr}s/{new_z_length}.*/{z_length}/" {output_gro}')
    run(f'gmx editconf -f {output_gro} -c -o {output_gro}')

def remove_ions_and_water_top(input_top, output_top, remove_resnames=None):
    if remove_resnames is None:
        remove_resnames = ["WAT", "SOL", "Na", "Cl","K", "Ca", "Mg", "Zn"]
    remove_resnames_set = set([x.upper() for x in remove_resnames])
    with open(input_top) as f:
        lines = f.readlines()
    with open(output_top, "w") as f:
        in_molecules = False
        for line in lines:
            stripped = line.strip()
            if stripped.lower().startswith("[ molecules ]"):
                in_molecules = True
                f.write(line)
                continue
            if in_molecules:
                if not stripped or stripped.startswith("["):
                    in_molecules = False
                    f.write(line)
                    continue
                parts = line.split()
                if parts and parts[0].upper() in remove_resnames_set:
                    continue
                f.write(line)
            else:
                f.write(line)

def get_pc_n31_box(gro_file):
    minz = 1e9
    maxz = -1e9
    with open(gro_file) as f:
        for l in f:
            #if len(l) > 44 and l[5:10].strip() == "PC" and l[10:15].strip() == "N31": #PE - N31, PS - N31, PH- -P31, PC - N31, PGR - C32, SPM - N31
            if len(l) > 44 and (l[5:10].strip(),l[10:15].strip()) in [("PE", 'N31'), ("PS", 'N31'),("PC", 'N31'), ("SPM", 'N31', ("PH-", 'P31'), ("PGR", 'C32'))]: #PE - N31, PS - N31, PH- -P31, PC - N31, PGR - C32, SPM - N31
                z = float(l[36:44])
                minz = min(minz, z)
                maxz = max(maxz, z)
    print(f"[INFO] PC N31 box min: {minz:.3f}, max: {maxz:.3f}")
    return minz, maxz

def remove_sol_in_zrange(input_gro, output_gro, minz, maxz, pad=0.0):
    minz -= pad
    maxz += pad
    removed = 0
    with open(input_gro) as f:
        lines = f.readlines()
    new_lines = [lines[0], lines[1]]
    natoms = 0
    i = 2
    while i < len(lines)-1:
        l = lines[i]
        if len(l) < 44:
            new_lines.append(l)
            i += 1
            continue
        resname = l[5:10].strip()
        resid = l[0:5]
        if resname == "SOL":
            j = i
            remove = False
            atoms = []
            while j < len(lines)-1 and lines[j][0:5] == resid and lines[j][5:10].strip() == "SOL":
                z = float(lines[j][36:44])
                if minz <= z <= maxz:
                    remove = True
                atoms.append(lines[j])
                j += 1
            if not remove:
                new_lines.extend(atoms)
                natoms += len(atoms)
            else:
                removed += 1
            i = j
        else:
            new_lines.append(l)
            natoms += 1
            i += 1
    new_lines.append(lines[-1])
    new_lines[1] = f"{natoms}\n"
    with open(output_gro, "w") as f:
        f.writelines(new_lines)
    print(f"[INFO] Wrote {output_gro} (removed {removed} SOL molecules in z=[{minz:.2f}, {maxz:.2f}])")

def count_sol_molecules(gro_file):
    count = 0
    last_resid = None
    with open(gro_file) as f:
        for line in f:
            if len(line) > 10 and line[5:10].strip() == "SOL":
                resid = line[0:5]
                if resid != last_resid:
                    count += 1
                    last_resid = resid
    return count

def update_sol_count_top(input_top, output_top, new_sol_count):
    with open(input_top) as f:
        lines = f.readlines()
    with open(output_top, "w") as f:
        in_molecules = False
        for line in lines:
            if line.strip().lower().startswith("[ molecules ]"):
                in_molecules = True
                f.write(line)
                continue
            if in_molecules:
                if line.strip() == "" or line.strip().startswith("["):
                    in_molecules = False
                    f.write(line)
                    continue
                parts = line.split()
                if parts and parts[0] == "SOL":
                    f.write(f"SOL     {new_sol_count}\n")
                else:
                    f.write(line)
            else:
                f.write(line)

def rename_sol_to_wat_gro(input_gro, output_gro):
    with open(input_gro) as f:
        lines = f.readlines()
    with open(output_gro, "w") as f:
        for l in lines:
            if len(l) > 10 and l[5:10].strip() == "SOL":
                l = l[:5] + "WAT" + l[8:]
            f.write(l)

def rename_sol_to_wat_top(input_top, output_top):
    with open(input_top) as f:
        lines = f.readlines()
    with open(output_top, "w") as f:
        for line in lines:
            parts = line.split()
            if parts and parts[0] == "SOL":
                line = line.replace("SOL", "WAT", 1)
            f.write(line)

def make_ndx_for_wat(grofile, ndxfile):
    cmd = f"gmx make_ndx -f {grofile} -o {ndxfile}"
    process = subprocess.Popen(cmd.split(), stdin=subprocess.PIPE, text=True)
    process.communicate("r WAT\nq\n")
    print(f"[INFO] Created {ndxfile} with group WAT")

def run_softcore_minimization(input_gro, input_top, min_mdp, output_tpr):
    run(f'export GMX_MAXCONSTRWARN=-1; gmx grompp -f {min_mdp} -r {input_gro} -c {input_gro} -p {input_top} -o {output_tpr} -maxwarn 3; gmx mdrun -deffnm softcore -v')
    run(f'gmx grompp -f {default_mdp} -r softcore.gro -c softcore.gro -p {input_top} -o membrane.tpr -maxwarn 3')
    run(f'gmx mdrun -deffnm membrane -v')
    run(f'unset GMX_MAXCONSTRWARN')

def prepare_once(base="system", min_mdp=None):
    #remove_ions_and_water_gro(input_gro, intermediate_gro)
    #remove_ions_and_water_top(input_top, intermediate_top)
    run(f'cp {input_gro} {intermediate_gro}')
    run(f'cp {input_top} {intermediate_top}')
    resize_box(intermediate_gro, resized_gro)
    run_softcore_minimization(resized_gro, intermediate_top, min_mdp, "softcore.tpr")
    print("[INFO] Preparation done. membrane.gro and noions.top ready.")

def fix_topol():
    protein = ['CYS','ASP','SER','GLN','LYS','ILE','PRO','THR','PHE','ASN','GLY','HIS','LEU','ARG','TRP','ALA','VAL','GLU','TYR','MET', 'GLH','NME','HIE','ACE']
    lipid = ['PA', 'PH-', 'AR', 'PS', 'ST', 'MY', 'PC', 'DHA', 'SA', 'SPM', 'LAL', 'PGR', 'OL', 'PE']
    nucleic = ['A','C','G', 'U', 'T']
    lipid_residues = {"DAPC": ["AR", "PC", "AR"],
    "DLPC": ["LAL", "PC", "LAL"],
    "DLPG": ["LAL", "PGR", "LAL"],
    "DMPC": ["MY", "PC", "MY"],
    "DMPG": ["MY", "PGR", "MY"],
    "DOPC": ["OL", "PC", "OL"],
    "DOPG": ["OL", "PGR", "OL"],
    "DOPS": ["OL", "PS", "OL"],
    "DPPC": ["PA", "PC", "PA"],
    "DPPG": ["PA", "PGR", "PA"],
    "DSPC": ["ST", "PC", "ST"],
    "DSPG": ["ST", "PGR", "ST"],
    "POPA": ["PA", "PH-", "OL"],
    "POPC": ["PA", "PC", "OL"],
    "POPE": ["PA", "PE", "OL"],
    "POPG": ["PA", "PGR", "OL"],
    "POPS": ["PA", "PS", "OL"],
    "PSM": ["PA", "SPM", "SA"],
    "SDPC": ["ST", "PC", "DHA"],
    "SSM": ["ST", "SPM", "SA"]}
    run(f'mv ff.itp toppar/')
    run(f'sed -i "s/ff/toppar\/ff/" topol.top')
    with open('topol.top') as f:
        lines = f.readlines()
    systems = []
    for line in lines:
        l = line.split()
        if len(l) == 0:
            break
        if l[0] == '#include':
            if l[1].split('/')[1][:6] == 'system':
                print(l[1].split('/')[1][:-1])
                systems.append(l[1][1:-1])
    system_types = {}
    number_protein = 1
    number_nucleic = 1
    for idx, system in enumerate(systems):
        #print(idx, system)
        with open(system) as f:
            lines = f.readlines()
            atoms = False
            residues = []
            for line in lines:
                # print(line[:6] )
                if atoms:
                    if line.startswith('\n'):
                        atoms = False
                        residues = sorted(list(set(residues)))
                        for l, r in lipid_residues.items():
                            #print(l,r, residues, sorted(set(r)))
                            if residues == sorted(set(r)):
                                system_types[idx+1] = l
                        break
                    res = line.split()[3]
                    if res in protein:
                        system_types[idx+1] = f'protein{number_protein}'
                        number_protein += 1
                        break
                    elif res in nucleic:
                        system_types[idx+1] = f'nucleic{number_nucleic}'
                        number_nucleic += 1
                        break
                    else:
                        residues.append(res)
                if line[:6] == ';   nr':
                    atoms = True
    for system, system_type in system_types.items():
        print(system, system_type)
        run(f'mv toppar/system{system}.itp toppar/{system_type}.itp')
        run(fr'sed -i "s/system{system}\./{system_type}\./g" toppar/{system_type}.itp')
        run(fr'sed -i "s/system{system}\./{system_type}\./g" topol.top')
        run(f'sed -i "s/system{system} /{system_type} /g" toppar/{system_type}.itp')
        run(f'sed -i "s/system{system} /{system_type} /g" topol.top')


def fix_itp(base):
    with open('topol.top') as f:
        lines = f.readlines()
    systems = []
    for line in lines:
        l = line.split()
        if len(l) == 0:
            break
        if l[0] == '#include':
            if l[1].split('/')[1].split('.')[0][:7]in ['protein', 'nucleic']:
                systems.append(l[1][1:-1])


    with open(f'{base[:-5]}_breaks_ACE_NME.pdb') as f:
        correct = f.readlines()

    chain_idx = 0
    correct_residues = {}
    correct_residues[chain_idx] = []
    for idx, line in enumerate(correct):
        if line[:3] == 'TER':
            chain_idx += 1
            correct_residues[chain_idx] = []

        elif line[:4] in ['ATOM', 'HETA']:
            resid = int(line[22:27])
            resname = line[17:20].strip()
            chain = line[21]
            if len(correct_residues[chain_idx]) == 0:
                correct_residues[chain_idx].append((resname, resid, chain))
            elif (resname, resid, chain) != correct_residues[chain_idx][-1]:
                correct_residues[chain_idx].append((resname, resid, chain))

    for chain_idx, system in enumerate(systems):
        with open(system) as f:
            lines = f.readlines()
        new_lines = ''

        atoms = False
        residues = []
        nxt = False
        res_idx = -1
        for line in lines:
            if atoms:
                if line.startswith('\n'):
                    atoms = False
                    residues = sorted(list(set(residues)))
                    new_lines += line
                elif line.startswith('; residue'):
                    res_idx += 1
                    resid = line.split()[2]
                    residue = line.split()[3]
                    if 'protein' in system:
                        try:
                            new_lines += line[:10] + f'{correct_residues[chain_idx][res_idx][1]:>4} {residue:>3}' + line[18:]
                        except:
                            chain_idx += 1
                            res_idx = 0
                            new_lines += line[:10] + f'{correct_residues[chain_idx][res_idx][1]:>4} {residue:>3}' + line[18:]
                    elif 'nucleic' in system:
                        new_lines += line
                        new_lines += line[:10] + f'{correct_residues[chain_idx][res_idx][1]:>4} {residue:<3}' + line[18:]
                    nxt = True


                else:
                    try:
                        resid, residue = line[19:31].split()
                        new_lines += line[:19] + f'{correct_residues[chain_idx][res_idx][1]:>4}{residue:>7}' + line[30:]
                        nxt = False
                    except:
                        chain_idx += 1
                        res_idx = 0
                        resid, residue = line[19:31].split()
                        new_lines += line[:19] + f'{correct_residues[chain_idx][res_idx][1]:>4}{residue:>7}' + line[30:]
                        nxt = False

            else:
                if line[:6] == ';   nr':
                    atoms = True

                new_lines += line

        with open(f"toppar/{system.split('/')[1]}", 'w') as f:
            f.write(new_lines)


def add_ter_rename_chains(base):

    # Lipids we should stop before (we don't assign chains/TERs into these)
    lipids = {
        'CHL','DLPC','DMPC','DPPC','DSPC','DOPC','POPC','POPE','DLPG','DMPG','DPPG',
        'DSPG','DOPG','POPG','DOPS','POPS','POPA','DAPC','SDPC','PSM','SSM'
    }
    # Common solvent/ion names to stop before (covers "no lipids → water" case)
    stop_before = {
        'SOL','WAT','HOH','TIP3','TIP3P','TIP4','TIP4P','TIP5','SPC','SPCE',
        'NA','NA+','CL','CL-','K','K+','MG','MG2','CA','CA2','ZN','ZN2',
        'SOD','POT','CLA'  # alternative ion labels
    }

    # --- 1) Parse non-solvent, non-lipid molecules (in order) from topol.top ---
    molecules = []  # list of (name, count)
    with open('topol.top') as f:
        in_mols = False
        for line in f:
            if line.startswith('; Compound'):
                in_mols = True
                continue
            if not in_mols:
                continue
            parts = line.split()
            if not parts:
                continue
            if parts[0].startswith('['):  # next section reached
                break
            name = parts[0]
            # stop when we hit first lipid or solvent/ion entry
            if name[:7] in lipids or name in stop_before:
                break
            try:
                count = int(parts[1])
            except Exception:
                continue
            molecules.append((name, count))

    # --- 2) Count atoms per molecule instance from its .itp ([ atoms ] rows) ---
    mol_atom_counts = {}
    for name, _ in molecules:
        atoms = 0
        with open(f'toppar/{name}.itp') as f:
            in_atoms = False
            for line in f:
                s = line.strip()
                if not s:
                    continue
                if s.startswith('['):
                    in_atoms = (s.lower().startswith('[ atoms ]'))
                    continue
                if not in_atoms or s.startswith(';'):
                    continue
                # Count actual atom rows (first token is an integer index)
                tok0 = s.split(None, 1)[0]
                if tok0.isdigit():
                    atoms += 1
        if atoms == 0:
            raise RuntimeError(f"No atoms parsed in IR0/r1/toppar/{name}.itp")
        mol_atom_counts[name] = atoms

    # Flatten per-instance list for the non-solvent/non-lipid region
    nonlipid_instances = []
    for name, count in molecules:
        for _ in range(count):
            nonlipid_instances.append((name, mol_atom_counts[name]))
    total_nonlipid_atoms = sum(n for _, n in nonlipid_instances)

    # --- helper: single-char chain IDs, cycle after 62 ---
    def next_chain_letter(idx):
        alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
        return alphabet[idx % len(alphabet)]

    with open(f'{base}.pdb') as f:
        pdb_lines = f.readlines()

    out = []
    chain_idx = 0
    current_chain_id = next_chain_letter(chain_idx)
    atoms_seen_in_instance = 0
    inst_i = 0
    atoms_in_this_instance = nonlipid_instances[0][1] if nonlipid_instances else 0

    prev_resid = None
    prev_is_atom = False

    is_atom = re.compile(r'^(ATOM  |HETATM)')

    def parse_resid(line):
        try:
            return int(line[22:26])
        except Exception:
            return None

    atom_counter_total = 0  # ATOM/HETATM ac-s the whole PDB

    for line in pdb_lines:
        if not is_atom.match(line):
            out.append(line)
            continue

        atom_counter_total += 1
        within_nonlipid = atom_counter_total <= total_nonlipid_atoms

        if within_nonlipid:
            resid = parse_resid(line)

            # Break only when residue number decreases or jumps by >1
            chain_break = (
                prev_is_atom and prev_resid is not None and resid is not None and
                (resid < prev_resid or resid - prev_resid > 1)
            )

            # Start of next instance: trigger TER before writing its first atom
            end_of_instance = (atoms_seen_in_instance + 1 > atoms_in_this_instance)

            if chain_break or end_of_instance:
                out.append('TER\n')
                chain_idx += 1
                current_chain_id = next_chain_letter(chain_idx)
                if end_of_instance:
                    inst_i += 1
                    if inst_i < len(nonlipid_instances):
                        atoms_in_this_instance = nonlipid_instances[inst_i][1]
                        atoms_seen_in_instance = 0
                prev_resid = None  # reset across boundary

            # write ATOM with chain ID in column 22 (0-based 21)
            new_line = f"{line[:21]}{current_chain_id}{line[22:]}"
            out.append(new_line)

            prev_resid = resid
            prev_is_atom = True
            atoms_seen_in_instance += 1

            # --- NEW: ensure a final TER after the very last non-lipid atom ---
            if atom_counter_total == total_nonlipid_atoms:
                out.append('TER\n')

        else:
            # Past the protein/ligand region: copy solvent/lipids unchanged
            out.append(line)
            prev_is_atom = True

    #return ''.join(out)
    with open(f'{base}.pdb','w') as f:
        f.write(''.join(out))


def resolvate_only(base="system", mdp="mdps/step6.0_minimization.mdp", water='OPC', ions="Na+,Cl-", conc="0.15"):
    water_type = {'OPC': 'tip4p', 'TIP3P': 'spc216', 'TIP4PEW': 'tip4p'}
    run(f"gmx solvate -cp membrane.gro -cs {water_type[water.upper()]}.gro -o {resolvated_gro} -p noions.top") ## change water model to user specified
    minz, maxz = get_pc_n31_box(resolvated_gro)
    remove_sol_in_zrange(resolvated_gro, cleaned_gro, minz, maxz)
    num_sol = count_sol_molecules(cleaned_gro)
    update_sol_count_top("noions.top", cleaned_top, num_sol)
    rename_sol_to_wat_gro(cleaned_gro, gro_wat)
    rename_sol_to_wat_top(cleaned_top, top_wat)
    make_ndx_for_wat(gro_wat, ndx)
    pion, nion = ions.split(',')
    run(f"gmx grompp -f {mdp} -r {gro_wat} -c {gro_wat} -p {top_wat} -o {tpr} -maxwarn 3")
    run(f'echo WAT | gmx genion -s {tpr} -p {top_wat} -pname {pion} -nname {nion} -neutral -conc {conc} -o {base}.gro -n {ndx}') ## Change to ions as per user preference
    run(f'cp {top_wat} topol.top')
    fix_topol()
    fix_itp(base)
    run(' '.join([sys.executable, "-m", "aaby.scripts.generate_restraints", args.mol2]))
    run(f"gmx grompp -f {mdp} -r {base}.gro -c {base}.gro -p topol.top -o pbc.tpr -maxwarn 3")
    #run(f'echo 0 | gmx trjconv -f {base}.gro -s pbc.tpr -o {base}.gro -pbc whole')
    run(f'echo 1 0 | gmx trjconv -f {base}.gro -s pbc.tpr -o {base}.gro -pbc res -center; echo 0 | gmx trjconv -f {base}.gro -s pbc.tpr -o {base}.gro -pbc whole')
    run(f'gmx editconf -f {base}.gro -o {base}.pdb -label X')
    add_ter_rename_chains(base)
    print(f"[DONE] See {base}.gro, {base}.pdb, and {base}.top for output.")

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--replicas", type=int, default=1, help="Number of replicas to run (default: 1)")
    parser.add_argument("--base", type=str, default="system", help="Base name for final output (default: system)")
    parser.add_argument('--water', default='OPC', help='Which water model to use (default: OPC) (Options: OPC, TIP3P, TIP4PEW)')
    parser.add_argument('--ions', default='Na+,Cl-', help='Which ions to use for solvation')
    parser.add_argument('--conc', default='0.15', help='Which ion concentration to build')
    parser.add_argument('--Z', default='10', help='Which Z height the box needs')
    parser.add_argument('--mol2', default=None, help='Is there a ligand in the system')
    # parser.add_argument('--membrane', default=False, help='If there is a membrane in the system')
    parser.add_argument('--membrane',action='store_true',help='If there is a membrane in the system')
    args = parser.parse_args()

    if args.membrane:
        default_mdp_path = MDPS_DIR / "MEMBRANE" / "step6.0_minimization.mdp"
        min_mdp_path     = MDPS_DIR / "MEMBRANE" / "step5_mini.mdp"
    else:
        default_mdp_path = MDPS_DIR / "SOLUTION" / "step6.0_minimization.mdp"
        min_mdp_path     = MDPS_DIR / "SOLUTION" / "step5_mini.mdp"

    default_mdp = str(default_mdp_path)
    min_mdp     = str(min_mdp_path)

    if args.replicas == 1:
        prepare_once(base=args.base, min_mdp=min_mdp)
        resolvate_only(base=args.base, mdp=default_mdp, water=args.water, ions=args.ions, conc=args.conc)

    else:
        prepare_once(base=args.base, min_mdp=min_mdp)
        for i in range(1, args.replicas + 1):
            rdir = f"r{i}"
            os.makedirs(rdir, exist_ok=True)
            shutil.copy("membrane.gro", os.path.join(rdir, "membrane.gro"))
            shutil.copy("noions.top", os.path.join(rdir, "noions.top"))

            # Copy mdps
            #shutil.copytree("../mdps", os.path.join(rdir, "mdps"), dirs_exist_ok=True)

            # Copy toppar (force field includes)
            shutil.copytree("toppar", os.path.join(rdir, "toppar"), dirs_exist_ok=True)

            # Copy ff.itp if present
            if os.path.exists("ff.itp"):
                shutil.copy("ff.itp", os.path.join(rdir, "ff.itp"))

            run(f'cp {args.base[:-5]}_breaks_ACE_NME.pdb r{i}')

            print(f"\n[INFO] Running replica {i} in {rdir}")
            os.chdir(rdir)
            try:
                #local_mdp = "mdps/step6.0_minimization.mdp"default_mdp
                resolvate_only(base=args.base, mdp=default_mdp, water=args.water, ions=args.ions, conc=args.conc)
            finally:
                os.chdir("..")



import sys

def add_ter_on_chain_change(infile, outfile):
    with open(infile) as fin:
        lines = fin.readlines()

    out_lines = []
    last_chain = None
    last_resnum = None

    for i, line in enumerate(lines):
        if line.startswith(("ATOM", "HETATM")):
            chain = line[21]
            resnum = line[22:26]
            if last_chain is None:
                last_chain = chain
            elif chain != last_chain:
                # Insert TER before this line
                ter_line = f"TER   {line[6:11]}      {last_resname} {last_chain}{last_resnum}\n"
                out_lines.append(ter_line)
                last_chain = chain
            last_resname = line[17:20]
            last_resnum = resnum
        out_lines.append(line)

    # Add TER at the end if the last line was an atom
    if lines and lines[-1].startswith(("ATOM", "HETATM")):
        ter_line = f"TER   {lines[-1][6:11]}      {last_resname} {last_chain}{last_resnum}\n"
        out_lines.append(ter_line)

    with open(outfile, "w") as fout:
        fout.writelines(out_lines)

if __name__ == "__main__":
    if len(sys.argv) != 3:
        print("Usage: python add_ter_on_chain_change.py input.pdb output.pdb")
        sys.exit(1)
    add_ter_on_chain_change(sys.argv[1], sys.argv[2])




import sys
from pathlib import Path
import random

def make_kicked_coords(x, y, z, direction=1):
    """Add a random small displacement in each direction, direction = +1 or -1 for NME/ACE."""
    # direction: +1 for NME, -1 for ACE
    dx = direction * (1.2 + random.uniform(0.1, 0.5))
    dy = random.uniform(-0.4, 0.4)
    dz = random.uniform(-0.4, 0.4)
    return x + dx, y + dy, z + dz

def make_ace_atom(atom_num, chain, first_atom_line):
    # Extract coordinates from the first atom
    x = float(first_atom_line[30:38])
    y = float(first_atom_line[38:46])
    z = float(first_atom_line[46:54])
    xn, yn, zn = make_kicked_coords(x, y, z, direction=-1)
    return (
        f"ATOM  {atom_num:5d}  C   ACE {chain}{first_atom_line[22:26]}{first_atom_line[26:30]}"
        f"{xn:8.3f}{yn:8.3f}{zn:8.3f}  1.00  0.00\n"
    )

def make_nme_atom(atom_num, chain, last_atom_line):
    # Extract coordinates from the last atom
    x = float(last_atom_line[30:38])
    y = float(last_atom_line[38:46])
    z = float(last_atom_line[46:54])
    xn, yn, zn = make_kicked_coords(x, y, z, direction=1)
    return (
        f"ATOM  {atom_num:5d}  C   NME {chain}{last_atom_line[22:26]}{last_atom_line[26:30]}"
        f"{xn:8.3f}{yn:8.3f}{zn:8.3f}  1.00  0.00\n"
    )

def main():
    if len(sys.argv) != 3:
        print("Usage: python add_ace_nme.py <pdb_file> <chainids_comma_separated>")
        sys.exit(1)

    pdb_file = Path(sys.argv[1])
    chains = set(sys.argv[2].split(","))

    with open(pdb_file) as f:
        lines = f.readlines()

    # Find highest atom number in the PDB to continue numbering
    max_atom_num = 0
    for l in lines:
        if l.startswith(("ATOM", "HETATM")):
            try:
                max_atom_num = max(max_atom_num, int(l[6:11]))
            except Exception:
                pass

    # Map: chain -> [atom_lines_indices]
    chain_indices = {c: [] for c in chains}
    for i, l in enumerate(lines):
        if l.startswith(("ATOM", "HETATM")):
            c = l[21]
            if c in chains:
                chain_indices[c].append(i)

    # Prepare insertions
    insertions = []
    for chain, indices in chain_indices.items():
        if not indices:
            print(f"Warning: chain {chain} not found.")
            continue
        first_idx = indices[0]
        last_idx = indices[-1]
        max_atom_num += 1
        ace_atom = make_ace_atom(max_atom_num, chain, lines[first_idx])
        max_atom_num += 1
        nme_atom = make_nme_atom(max_atom_num, chain, lines[last_idx])

        insertions.append((first_idx, ace_atom))       # ACE goes *before* first atom
        insertions.append((last_idx + 1, nme_atom))    # NME goes *after* last atom

    # Insert from back so indices don't shift
    for idx, new_line in sorted(insertions, reverse=True):
        lines.insert(idx, new_line)

    # Write output
    out_file = pdb_file.with_name(pdb_file.stem + "_ACE_NME.pdb")
    with open(out_file, "w") as f:
        f.writelines(lines)
    print(f"Wrote {out_file}")

if __name__ == "__main__":
    main()



# from setuptools import setup, find_packages

# setup(
#     name="aaby",
#     version="0.1.0",
#     description="Automated AMBER System Builder Workflow",
#     author="Your Name",
#     packages=find_packages(),
#     include_package_data=True,
#     package_data={"aaby": ["data/tleap.in"]},
#     install_requires=[
#         "pyyaml",
#         # "importlib_resources",  # For Python <3.9
#     ],
#     entry_points={
#         "console_scripts": [
#             "aaby = aaby.main:main",
#         ],
#     },
# )
from setuptools import setup, find_packages

setup(
    name="aaby",
    version="0.1.2",
    description="Automated AMBER System Builder Workflow",
    author="Amanda D. Stange",
    packages=find_packages(),
    include_package_data=True,  # needed so [tool.setuptools.package-data] is honoured
    entry_points={
        "console_scripts": [
            "aaby = aaby.main:main",
        ],
    },
)



#!/usr/bin/env python3
import time
start_time = time.time()

import argparse
import subprocess
from pathlib import Path
import sys
import re
import os
import shutil
import shlex
import yaml
from openbabel import openbabel


# Package root: .../site-packages/aaby
PACKAGE_ROOT = Path(__file__).resolve().parents[1]

# Data/resources live under aaby/resources/...
RESOURCES_DIR = PACKAGE_ROOT / "resources"
MODELS_DIR = RESOURCES_DIR / "models"
MDPS_DIR = RESOURCES_DIR / "mdps"
TOPPAR_DIR = RESOURCES_DIR / "toppar"
COBY_TOPPAR_DIR = RESOURCES_DIR / "COBY_toppar"
TLEAP_DIR = RESOURCES_DIR / "tleap"

print("AABY package root:", PACKAGE_ROOT)

# Global variable for logfile
LOGFILE = None


class Tee:
    """Duplicate output to console and file(s)."""
    def __init__(self, *files):
        self.files = files
    def write(self, obj):
        for f in self.files:
            f.write(obj)
            f.flush()
    def flush(self):
        for f in self.files:
            f.flush()

#def run(cmd, shell=True, check=True):
#    if isinstance(cmd, list):
#        print(f"Running: {' '.join(str(x) for x in cmd)}")
#    else:
#        print(f"Running: {cmd}")
#    subprocess.run(cmd, shell=shell, check=check)






# def run(cmd, check=True, shell=None):
#     """Run a shell command or Python script with logging.

#     Args:
#         cmd (str | list): Command string or list of arguments.
#         check (bool): Raise error on non-zero exit if True.
#         shell (bool | None):
#             - None → auto-detect (safe default)
#             - True/False → override
#     """
#     global LOGFILE

#     # Detect if we need shell
#     if isinstance(cmd, str):
#         if shell is None:
#             # auto-detect dangerous shell operators
#             if re.search(r"[><|;&]", cmd):
#                 shell = True
#             else:
#                 shell = False
#         cmd_list = cmd if shell else shlex.split(cmd)
#     else:
#         # Already a list
#         cmd_list = list(map(str, cmd))
#         if shell is None:
#             shell = False

#     # Pretty print
#     cmd_str = cmd if isinstance(cmd, str) else " ".join(cmd_list)
#     print(f"Running: {cmd_str}")

#     if LOGFILE:
#         with open(LOGFILE, "a") as f:
#             subprocess.run(cmd_list, check=check, shell=shell,
#                            stdout=Tee(sys.stdout, f),
#                            stderr=Tee(sys.stderr, f))
#     else:
#         subprocess.run(cmd_list, check=check, shell=shell)

def run(cmd, check=True, shell=None):
    """Run a shell command or Python script with optional logging."""

    global LOGFILE

    # Detect if we need shell
    if isinstance(cmd, str):
        if shell is None:
            # auto-detect dangerous shell operators
            if re.search(r"[><|;&]", cmd):
                shell = True
            else:
                shell = False
        cmd_list = cmd if shell else shlex.split(cmd)
    else:
        cmd_list = list(map(str, cmd))
        if shell is None:
            shell = False

    cmd_str = cmd if isinstance(cmd, str) else " ".join(cmd_list)
    print(f"Running: {cmd_str}")

    if LOGFILE:
        # Run and capture output
        result = subprocess.run(
            cmd_list,
            check=check,
            shell=shell,
            text=True,
            capture_output=True,
        )

        # Tee to terminal
        if result.stdout:
            print(result.stdout, end="")
        if result.stderr:
            print(result.stderr, end="", file=sys.stderr)

        # Append to logfile
        with open(LOGFILE, "a") as f:
            f.write(f"\n$ {cmd_str}\n")
            if result.stdout:
                f.write(result.stdout)
            if result.stderr:
                f.write(result.stderr)

        return result
    else:
        # No logging: just run normally, inherit parent stdout/stderr
        return subprocess.run(cmd_list, check=check, shell=shell)

def add_ter_rename_chains(pdb, out_ter_rename_chains):

    run(f"gmx editconf -f {pdb} -o {str(pdb).split('.')[0]}_gmx.pdb")
    with open(f"{str(pdb).split('.')[0]}_gmx.pdb") as f:
        lines = f.readlines()
    resids = []
    resids_unique = []
    chainids = []
    new_chain = []
    TERs = []
    idxs = []
    isATOM = False
    i = 0
    for idx, line in enumerate(lines):
        if line[:4] in ['ATOM', 'HETA']:
            resid = int(line[22:27])
            chainid = line[21]
            if i == 0:
                resids_unique.append(resid)
                new_chain.append([chainid, idx])
                i += 1
            if resids_unique[-1] != resid:
                resids_unique.append(resid)
            if new_chain[-1][0] != chainid:
                new_chain.append([chainid, idx])
            resids.append(resid)
            chainids.append(chainid)
            idxs.append(idx)

        if line[:3] in ['TER']:
            TERs.append(idx)
    new_TERs = []

    for i in range(1,len(resids)):
        if abs(resids[i] - resids[i-1]) > 1:
            if idxs[i] - idxs[i - 1] != 2:

                new_TERs.append(idxs[i] + len(new_TERs))
                print(resids[i-1], resids[i], chainids[i-1], chainids[i], idxs[i-1], idxs[i])

    for idx in new_TERs:
        lines.insert(idx, 'TER\n')



    TERs = []
    for idx, line in enumerate(lines):
        if line == 'TER\n':
            TERs.append(idx)



    chainids = {}
    chains = range(65, 65+28)
    TERs = [0] + TERs

    for idx in range(len(TERs)-1):
        chainids[chr(chains[idx])] = range(TERs[idx]+1, TERs[idx+1]+1)


    new_lines = ''
    for idx, line in enumerate(lines):
        if line[:4] in ['ATOM', 'HETA']:
            for k, v in chainids.items():
                if idx in v:
                    chain = k
                    break
            new_lines += f'{line[:21]}{chain}{line[22:]}'
        else:
            new_lines += line
    with open(out_ter_rename_chains, "w") as f:
        f.write("".join(new_lines))





def insert_ssbonds_into_tleap(tleap_in_path='tleap.in', ssbond_path='tleap_SSBONDs.txt'):
    tleap_in = Path(tleap_in_path)
    ssbonds = Path(ssbond_path)
    if not tleap_in.exists() or not ssbonds.exists():
        print(f"Warning: {tleap_in} or {ssbonds} does not exist. Skipping SSBOND insertion.")
        return
    with tleap_in.open() as f:
        lines = f.readlines()
    for idx, line in enumerate(lines):
        if 'loadpdb' in line:
            loadpdb_idx = idx
            break
    else:
        print("Could not find 'loadpdb' in tleap.in!")
        return
    with ssbonds.open() as f:
        ssbond_lines = f.readlines()
    new_lines = lines[:loadpdb_idx + 1] + ssbond_lines + lines[loadpdb_idx + 1:]
    with tleap_in.open('w') as f:
        f.writelines(new_lines)
    print(f"Inserted {ssbonds} after loadpdb in {tleap_in}")

def make_coby_ter_pdb(coby_pdb, pre_coby_pdb, coby_ter_pdb):
    """
    Insert TER records into the COBY PDB wherever a TER occurred in the pre-COBY PDB,
    by matching residue numbers. Assumes all chains become A in COBY output.
    """
    # 1. Find residue numbers before each TER in the pre-COBY file
    ter_resids = []
    prev_resid = None
    with open(pre_coby_pdb) as f:
        for line in f:
            if line.startswith(("ATOM", "HETATM")):
                prev_resid = int(line[22:26])
            elif line.startswith("TER"):
                if prev_resid is not None:
                    ter_resids.append(prev_resid)
                prev_resid = None

    # 2. Parse COBY PDB, insert TER after each matching resid
    output_lines = []
    last_resid = None
    last_atomnum = None
    last_resname = None

    for i, line in enumerate(open(coby_pdb)):
        output_lines.append(line)
        if line.startswith(("ATOM", "HETATM")):
            resid = int(line[22:26])
            atomnum = int(line[6:11])
            resname = line[17:20]
            if last_resid is not None and last_resid in ter_resids and resid != last_resid:
                # Insert TER *after* last atom of the pre-TER residue
                ter_line = f"TER   {last_atomnum:5d}      {last_resname} A{last_resid:>4}\n"
                output_lines.insert(-1, ter_line)  # Insert before the current line
            last_resid = resid
            last_atomnum = atomnum
            last_resname = resname

    # Also handle case where the last residue should get a TER (optional)
    # (Not usually needed; leave as is)

    with open(coby_ter_pdb, 'w') as f:
        f.writelines(output_lines)
    print(f"Wrote merged COBY/protein file with TERs: {coby_ter_pdb}")


def auto_detect_types(water_model='OPC'):
    #input4amber.pdb
    # source leaprc.protein.ff19SB
    # source leaprc.water.opc
    # source leaprc.lipid21
    #sed -i '1s/^/task goes here\n/' todo.txt

    forcefields = {'protein': 'protein.ff19SB', 'lipid': 'lipid21', 'dna': 'DNA.OL24', 'rna': 'RNA.OL3', 'water': {'OPC': 'water.opc', 'TIP3P': 'water.tip3p', 'TIP4PEW': 'water.tip4pew'}}

    protein = ['CYS','ASP','SER','GLN','LYS','ILE','PRO','THR','PHE','ASN','GLY','HIS','LEU','ARG','TRP','ALA','VAL','GLU','TYR','MET', 'GLH','NME','HIE','ACE', 'HID', 'CYX']
    lipid = ['PC', 'PA', 'OL', 'CHL']
    lipid = ['PA', 'PH-', 'AR', 'PS', 'ST', 'MY', 'PC', 'DHA', 'SA', 'SPM', 'LAL', 'PGR', 'OL', 'PE','CHL']
    nucleic = ['A','C','G']
    rna = ['U']
    dna = ['T']
    # ions = ['NA']
    with open('input4amber.pdb', 'r') as f:
        lines = f.readlines()

    resnames = []

    ff_types = []

    for line in lines:

        l = line.split()

        if l[0] == 'END':
            break

        if l[0] in ['ATOM', 'HETATM']:
            resnames.append(l[3])

    resnames = list(set(resnames))



    for resname in resnames:
        if resname in protein:
            if 'protein' not in ff_types:
                ff_types.append('protein')
        elif resname in lipid:
            if 'lipid' not in ff_types:
                ff_types.append('lipid')
        # elif resname in nucleic:
        #     if 'nucleic' not in ff_types:
        #         ff_types.append('nucleic')
        elif resname in rna:
            if 'rna' not in ff_types:
                ff_types.append('rna')
        elif resname in dna:
            if 'dna' not in ff_types:
                ff_types.append('dna')
        else:
            print('type not found', resname, ff_types)

    leaprc = ''

    for ff_type in sorted(ff_types, key=lambda x: len(x), reverse=True):
        leaprc += f'source leaprc.{forcefields[ff_type]}\\n'

    leaprc += f"source leaprc.{forcefields['water'][water_model.upper()]}\\n"

    return leaprc

def substitute_protein_pdb(args_list, protein_pdb):
    return [str(x).replace("{protein_pdb}", str(protein_pdb)) for x in args_list]

def autodetect_amber_files():
    prmtop_candidates = sorted(Path('.').glob('*.prmtop'), key=lambda f: f.stat().st_mtime, reverse=True)
    inpcrd_candidates = sorted(Path('.').glob('*.inpcrd'), key=lambda f: f.stat().st_mtime, reverse=True)
    if not prmtop_candidates or not inpcrd_candidates:
        raise RuntimeError("tleap failed to produce .prmtop or .inpcrd")
    prmtop = prmtop_candidates[0]
    inpcrd = inpcrd_candidates[0]
    print(f"Using prmtop: {prmtop}")
    print(f"Using inpcrd: {inpcrd}")
    return prmtop, inpcrd

def infer_element(line):
    TWO_LETTER_ELEMS = {
    "Cl","Br","Na","Mg","Al","Si","Li","Be","Ca","Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu",
    "Zn","Ga","Ge","As","Se","Kr","Rb","Sr","Zr","Nb","Mo","Tc","Ru","Rh","Pd","Ag","Cd",
    "In","Sn","Sb","Te","Xe","Cs","Ba","La","Ce","Pr","Nd","Pm","Sm","Eu","Gd","Tb","Dy",
    "Ho","Er","Tm","Yb","Lu","Hf","Ta","W","Re","Os","Ir","Pt","Au","Hg","Tl","Pb","Bi",
    "Po","At","Rn"
}
    """Prefer PDB element field (cols 77–78). Else infer from atom name."""
    elem = line[76:78].strip() if len(line) >= 78 else ""
    if elem:
        e = elem.capitalize()
        return e if e in TWO_LETTER_ELEMS else e[:1]
    # Fallback from atom name field (cols 13–16)
    an = (line[12:16] if len(line) >= 16 else "").strip()
    if not an:
        return "X"
    if an[:2].capitalize() in ("Cl","Br"):
        return an[:2].capitalize()
    if an[:2].capitalize() in TWO_LETTER_ELEMS:
        return an[:2].capitalize()
    return an[0].upper()

def format_pdb_atom_name(name, element):
    """Return a 4-char atom-name field with PDB alignment rules."""
    if len(element) == 2:
        return name.ljust(4)[:4]      # 2-letter element -> left-justified
    else:
        return name.rjust(4)[-4:]      # 1-letter element -> right-justified

def rename_pdb4antechamber(
    inp, outp, cap=4, include_resnames=None, case_insensitive=True,
    renumber=False, renumber_start=1, update_conect=False
):
    """
    Rename atom names uniquely (per-residue) only for residues in include_resnames,
    and optionally renumber all atom serials in the full PDB (updates CONECT).

    Args
    ----
    inp : str        input PDB
    outp: str        output PDB
    cap : int        max atom-name length (default 4)
    include_resnames: {str} or list/tuple/set of resnames to affect; None = all
    case_insensitive: bool  compare resnames case-insensitively
    renumber : bool  renumber serials across entire file
    renumber_start : int    starting serial
    update_conect : bool    update CONECT lines to new serials when renumbering
    """
    # normalize filter set
    if include_resnames is None:
        filter_set = None
    else:
        rs = include_resnames if isinstance(include_resnames, (list, tuple, set)) else [include_resnames]
        filter_set = set(r.upper() if case_insensitive else r for r in rs)

    # First pass: rename (only filtered residues), collect lines
    out_lines = []
    counts = {}       # per-residue & element counters
    model_id = 0

    def resname_pass(line):
        rn = line[17:20]
        return rn.strip().upper() if case_insensitive else rn.strip()

    for raw in open(inp, "r"):
        line = raw.rstrip("\n")
        rec = line[:6]

        if rec.startswith("MODEL"):
            # try parse model number, else increment
            try:
                model_id = int(line[10:14].strip())
            except Exception:
                model_id += 1
            out_lines.append(line)
            continue

        if rec.startswith("ATOM  ") or rec.startswith("HETATM"):
            # ensure padding for safe slicing
            if len(line) < 80:
                line = line + " " * (80 - len(line))

            resname = resname_pass(line)
            do_rename = (filter_set is None) or (resname in filter_set)

            if do_rename:
                chain   = line[21]
                resseq  = line[22:26]
                icode   = line[26]
                elem    = infer_element(line)

                # per-residue + element key
                key = (chain, resseq, icode, resname, model_id, elem)
                counts[key] = counts.get(key, 0) + 1

                new = f"{elem}{counts[key]}"
                if len(new) > cap:
                    new = elem + str(counts[key])[: max(1, cap - len(elem))]

                name_field = format_pdb_atom_name(new, elem)
                line = line[:12] + name_field + line[16:]

                # normalize element field
                elem_field = elem.rjust(2)[:2]
                line = line[:76] + elem_field + line[78:]

            out_lines.append(line)
        else:
            out_lines.append(line)

    # If no renumbering, write and return
    if not renumber:
        with open(outp, "w") as g:
            g.write("\n".join(out_lines) + ("\n" if out_lines and out_lines[-1] != "" else ""))
        return

    # Second pass: renumber serials (ATOM/HETATM) and update CONECT
    # Build mapping old_serial -> new_serial
    serial_map = {}
    next_serial = renumber_start
    renum_lines = []

    for line in out_lines:
        rec = line[:6]
        if rec.startswith("ATOM  ") or rec.startswith("HETATM"):
            if len(line) < 80:
                line = line + " " * (80 - len(line))
            old_serial = int(line[6:11])
            serial_map[old_serial] = next_serial
            # write new serial into cols 7-11
            new_serial_field = f"{next_serial:5d}"
            line = line[:6] + new_serial_field + line[11:]
            next_serial += 1
        renum_lines.append(line)

    # Update CONECT if requested
    if update_conect:
        final_lines = []
        for line in renum_lines:
            if line.startswith("CONECT"):
                # CONECT + up to five serials in 5-wide fields
                # columns: 7-11 (atom) then 12-16, 17-21, 22-26, 27-31
                # We'll parse ints and rewrite via mapping (skip if not present)
                fields = [line[:6]]  # "CONECT"
                nums = []
                # grab 5-wide fields after col 6
                for i in range(6, len(line), 5):
                    chunk = line[i:i+5]
                    if chunk.strip().isdigit():
                        nums.append(int(chunk))
                    else:
                        nums.append(None)
                if nums:
                    mapped = []
                    for n in nums:
                        if n is None:
                            mapped.append("     ")
                        else:
                            mapped.append(f"{serial_map.get(n, n):5d}")
                    line = fields[0] + "".join(mapped)
                    # ensure newline
                final_lines.append(line)
            else:
                final_lines.append(line)
    else:
        final_lines = renum_lines

    with open(outp, "w") as g:
        g.write("\n".join(final_lines) + ("\n" if final_lines and final_lines[-1] != "" else ""))

def adjust_mol2_charges(input_file, output_file, target_charge):
    atoms = []
    in_atom_section = False
    target_charge = int(target_charge)
    total_charge = 0.0

    with open(input_file, "r") as f:
        lines = f.readlines()

    # Parse atoms and charges
    for i, line in enumerate(lines):
        if line.strip().startswith("@<TRIPOS>ATOM"):
            in_atom_section = True
            continue
        elif line.strip().startswith("@<TRIPOS>") and in_atom_section:
            in_atom_section = False
        if in_atom_section and line.strip():
            parts = line.split()
            atom_id, atom_name, x, y, z, atom_type, res_id, res_name, charge = parts[:9]
            charge = float(charge)
            atoms.append({
                "line_index": i,
                "parts": parts,
                "charge": charge,
                "name": atom_name,
                "type": atom_type.lower()
            })
            total_charge += charge

    # Define non-polar hydrogen types (Amber / Antechamber style)
    nonpolar_H_types = {"ha", "hc", "h1", "h2", "h3"}

    # Select only nonpolar hydrogens
    nonpolar_H = [a for a in atoms if a["type"] in nonpolar_H_types]
    if not nonpolar_H:
        raise ValueError("No nonpolar hydrogens found to distribute charge.")

    # Compute correction
    diff = target_charge - round(total_charge, 8)
    correction_per_H = diff / len(nonpolar_H)

    print(f"Original total charge: {total_charge:.8f}")
    print(f"Target charge: {target_charge}")
    print(f"Correction needed: {diff:.8f}")
    print(f"Distributing {correction_per_H:.8f} across {len(nonpolar_H)} nonpolar hydrogens")

    # Apply correction equally
    for h in nonpolar_H:
        h["charge"] += correction_per_H

    # Fix drift: adjust last hydrogen to hit exact target
    corrected_total = sum(a["charge"] for a in atoms)
    final_diff = target_charge - corrected_total
    nonpolar_H[-1]["charge"] += final_diff

    # Update atom charge fields with 8 decimals
    for atom in atoms:
        atom["parts"][8] = f"{atom['charge']:.8f}"
        lines[atom["line_index"]] = "{:>7} {:<9} {:>10} {:>10} {:>10} {:<4} {:>5} {:<4} {:>12}\n".format(
            atom["parts"][0], atom["parts"][1], atom["parts"][2], atom["parts"][3],
            atom["parts"][4], atom["parts"][5], atom["parts"][6], atom["parts"][7],
            atom["parts"][8]
        )

    # Write new mol2
    with open(output_file, "w") as f:
        f.writelines(lines)

    final_total = sum(a["charge"] for a in atoms)
    print(f"Adjusted mol2 written to {output_file}")
    print(f"Final total charge: {final_total:.8f}")


def antechamber(mol2=None, nc=None, input_pdb=None, chain=None):
    if nc is None:
        sys.exit("--nc option is not set. This is required for using the --antechamber option!")
    ## Lines taken from amber_geostad gcif_to_mol2
    #res = mol2.split('_')[0]
    if chain is None:
        run(f'grep {mol2} {input_pdb} > {mol2}.pdb')
    else:
        run(f'grep "{mol2} {chain}" {input_pdb} > {mol2}.pdb')
    # rename_pdb4antechamber(f'{mol2}.pdb', f'{mol2}_fix.pdb')
    #run(f'gmx editconf -f {mol2}.pdb -resnr 1 -o {mol2}.pdb')
    obConversion = openbabel.OBConversion()
    obConversion.SetInAndOutFormats("pdb", "mol2")
    mol = openbabel.OBMol()
    obConversion.ReadFile(mol, f"{mol2}.pdb")   # Open Babel will uncompress automatically
    obConversion.WriteFile(mol, f'{mol2}_obabel.mol2')
    # try:
    #     run(f'obabel {mol2}.pdb -O {mol2}_obabel.mol2')
    # except:
    #     sys.exit("obabel not installed. This is required for using the --antechamber option! Run 'sudo apt install openbabel' to continue")
    # run(f'sed -i "s/{mol2}.pdb/{mol2}/" {mol2}_obabel.mol2')
    # run(f'sed -i "s/{mol2}1/{mol2} /" {mol2}_obabel.mol2')
    #run(f"$AMBERHOME/bin/antechamber -i {mol2}_obabel.mol2 -fi mol2 -o {mol2}.mol2 -fo mol2 -bk comp_{mol2} -s 0 -dr no -nc {nc} -at gaff2 -c abcg2 -ek 'qm_theory=\"AM1\", maxcyc=1000, ndiis_attempts=700,'")
    run(f"antechamber -i {mol2}_obabel.mol2 -fi mol2 -o {mol2}.mol2 -fo mol2 -bk comp_{mol2} -s 0 -dr no -nc {nc} -at gaff2 -c bcc -ek 'qm_theory=\"AM1\", maxcyc=1000, ndiis_attempts=700,'")
    adjust_mol2_charges(f'{mol2}.mol2', f'{mol2}.mol2', nc)
    #run(f"$AMBERHOME/bin/parmchk2 -s 2 -i {mol2}.mol2 -f mol2 -o {mol2}.frcmod")
    run(f"parmchk2 -s 2 -i {mol2}.mol2 -f mol2 -o {mol2}.frcmod")
    run(f'/bin/rm -f ANTECH* ATOMTYP* sqm.*')
    tleap_ante = f"source leaprc.gaff2 \\n{mol2} = loadMol2 {mol2}.mol2 \\nloadAmberParams {mol2}.frcmod\\n"
    run(f'sed -i "1s/^/{tleap_ante}/" tleap.in')
    run(f'sed -i "1s/^/{tleap_ante}/" tleap_solv.in')



def main():
    global LOGFILE
    parser = argparse.ArgumentParser(description="AABY: End-to-end AMBER system builder from PDB")
    parser.add_argument('-f', '--input', required=True, help='Input PDB file')
    parser.add_argument('--chains', default="A", help='Comma-separated list of protein chains for ACE/NME')
    parser.add_argument('-r', '--replicas', type=int, default=1, help='Number of resolvated replicas')
    parser.add_argument('--ssbond', action='store_true', help='Renumber SSBOND.txt, convert CYS to CYX, insert ssbonds into tleap.in')
    parser.add_argument('--water', default='OPC', help='Which water model to use (default: OPC) (Options: OPC, TIP3P, TIP4PEW)')
    parser.add_argument('--ions', default='Na+,Cl-', help='Which ions to use for solvation (first positive then negative)')
    parser.add_argument('--conc', default='0.15', help='Which ion concentration to build')
    parser.add_argument('--antechamber', default=False, help='Option to use antechamber to parameterise ligands')
    parser.add_argument('--mol2', default=None, help='Name of the ligand to be parameterised')
    parser.add_argument('--mol2chain', default=None, help='If multiple copies of the same ligand specific chain for first one')
    parser.add_argument('--nc', default=None, help='Net charge of ligand')
    parser.add_argument('--log', nargs='?', const="AABY.log", help='Write all terminal output to a log file (default: AABY.log)')
    group = parser.add_mutually_exclusive_group()
    group.add_argument('--ph', type=float, help='pH for propka (protonation by predicted pKa)')
    group.add_argument('--protlist', type=str, help='Residue list file for direct protonation changes')
    parser.add_argument('--coby-yaml', type=str, help='YAML file specifying COBY arguments (use {protein_pdb} as placeholder)')
    parser.add_argument('--coby-args', nargs=argparse.REMAINDER, help='Arguments passed directly to COBY after this flag (use {protein_pdb} as placeholder)')
    args = parser.parse_args()

    # Setup logging if requested
    # if args.log:
    #     LOGFILE = args.log
    #     logfile = open(LOGFILE, "w")
    #     sys.stdout = Tee(sys.stdout, logfile)
    #     sys.stderr = Tee(sys.stderr, logfile)

    if args.log:
        LOGFILE = args.log
        # Start with an empty logfile
        with open(LOGFILE, "w") as f:
            f.write("AABY log\n")

    # Copy tleap templates from packaged resources to local working directory
    tleap_template = TLEAP_DIR / "tleap.in"
    tleap_local = Path("tleap.in")
    shutil.copy(tleap_template, tleap_local)

    tleap_solv_template = TLEAP_DIR / "tleap_solv.in"
    tleap_solv_local = Path("tleap_solv.in")
    shutil.copy(tleap_solv_template, tleap_solv_local)

    pdb = Path(args.input)
    base = pdb.stem

    if args.antechamber:
        out_renamedligandatoms = pdb.with_name(base + '_renameligandatoms.pdb')
        rename_pdb4antechamber(pdb, out_renamedligandatoms, include_resnames=args.mol2)
        # 0. Add TERs and rename chains
        out_ter_rename_chains = pdb.with_name(base + '_breaks.pdb')
        add_ter_rename_chains(out_renamedligandatoms, out_ter_rename_chains)
    else:
        # 0. Add TERs and rename chains
        out_ter_rename_chains = pdb.with_name(base + '_breaks.pdb')
        add_ter_rename_chains(pdb, out_ter_rename_chains)




    # 1. Add caps
    out_ace_nme = pdb.with_name(base + '_breaks_ACE_NME.pdb')
    #run(f'python {AABY_dir}/Scripts/add_ace_nme.py {out_ter_rename_chains} {args.chains}')
    run([sys.executable,"-m","aaby.scripts.add_ace_nme",str(out_ter_rename_chains),args.chains,])
    assert out_ace_nme.exists(), "add_ace_nme failed"

    # 2. Renumber
    out_renum = pdb.with_name(base + '_breaks_ACE_NME_re.pdb')
    run(f'gmx editconf -f {out_ace_nme} -resnr 1 -o {out_renum}')
    assert out_renum.exists(), "editconf renumber failed"

    # 3. SSBOND handling if requested
    if args.ssbond:
        #run(f'python {AABY_dir}/Scripts/re_ss.py {out_ace_nme}')
        run([sys.executable, "-m", "aaby.scripts.re_ss", str(out_ace_nme)])
        ssbonded_pdb = out_ace_nme.with_name(out_renum.stem + '_ssbond.pdb')
        assert ssbonded_pdb.exists(), "re_ss failed"
        working_pdb = ssbonded_pdb
        insert_ssbonds_into_tleap('tleap.in', 'tleap_SSBONDs.txt')
        insert_ssbonds_into_tleap('tleap_solv.in', 'tleap_SSBONDs.txt')
    else:
        working_pdb = out_renum

    # 4. Delete H (heavy atom only)
    out_heavy = working_pdb.with_name(working_pdb.stem + '_heavy.pdb')
    run(f'printf "del 0-100\n!a H*\n q\n" | gmx make_ndx -f {working_pdb} -o heavy.ndx')
    file_size = os.path.getsize('heavy.ndx')
    if file_size == 0:
        run(f'cp {working_pdb} {out_heavy}')
    else:
        run(f'gmx trjconv -f {working_pdb} -s {working_pdb} -n heavy.ndx -o {out_heavy}')
    assert out_heavy.exists(), "trjconv failed"

    # # 5. Protonation (by pH/pKa or residue list)
    # if args.ph is not None:
    #     pka_file = out_heavy.with_suffix('.pka')
    #     #run(f'python {AABY_dir}/Scripts/pka.py {out_heavy} {args.ph}')
    #     run([sys.executable, "-m", "aaby.scripts.pka", str(out_heavy), str(args.ph)])
    #     assert pka_file.exists(), "pka calculation failed"
    #     #run(f'python {AABY_dir}/Scripts/change_prot.py {pka_file} --ph {args.ph} -o mutate_residues.sh')
    #     run([sys.executable, "-m", "aaby.scripts.change_prot", str(pka_file), "--ph", str(args.ph), "-o", "mutate_residues.sh"])
    #     shutil.copy(out_heavy, out_heavy.with_name(out_heavy.stem + '_renamed.pdb'))
    #     renamed_pdb = out_heavy.with_name(out_heavy.stem + '_renamed.pdb')
    #     run(f'bash mutate_residues.sh {renamed_pdb}')
    # # elif args.protlist:
    # #     #run(f'python {AABY_dir}/Scripts/change_prot.py {args.protlist} --list -o mutate_residues.sh')
    # #     run([sys.executable, "-m", "aaby.scripts.change_prot", str(pka_file), "--ph", str(args.ph), "-o", "mutate_residues.sh"])
    # #     shutil.copy(out_heavy, out_heavy.with_name(out_heavy.stem + '_renamed.pdb'))
    # #     renamed_pdb = out_heavy.with_name(out_heavy.stem + '_renamed.pdb')
    # #     run(f'bash mutate_residues.sh {renamed_pdb}')
    # elif args.protlist:
    #     run([
    #         sys.executable,
    #         "-m",
    #         "aaby.scripts.change_prot",
    #         str(args.protlist),
    #         "--list",
    #         "-o",
    #         "mutate_residues.sh",
    #     ])
    #     shutil.copy(out_heavy, out_heavy.with_name(out_heavy.stem + "_renamed.pdb"))
    #     renamed_pdb = out_heavy.with_name(out_heavy.stem + "_renamed.pdb")
    #     run(f"bash mutate_residues.sh {renamed_pdb}")


    # 5. Protonation (by pH/pKa or residue list)
    renamed_pdb = None

    if args.ph is not None:
        # pH-driven protonation via PROPKA
        pka_file = out_heavy.with_suffix('.pka')
        run([sys.executable, "-m", "aaby.scripts.pka", str(out_heavy), str(args.ph)])
        assert pka_file.exists(), "pka calculation failed"

        run([
            sys.executable, "-m", "aaby.scripts.change_prot",
            str(pka_file), "--ph", str(args.ph), "-o", "mutate_residues.sh"
        ])

        renamed_pdb = out_heavy.with_name(out_heavy.stem + '_renamed.pdb')
        shutil.copy(out_heavy, renamed_pdb)
        run(f"bash mutate_residues.sh {renamed_pdb}")

    elif args.protlist:
        # Explicit residue list for protonation changes
        run([
            sys.executable, "-m", "aaby.scripts.change_prot",
            str(args.protlist), "--list", "-o", "mutate_residues.sh"
        ])

        renamed_pdb = out_heavy.with_name(out_heavy.stem + '_renamed.pdb')
        shutil.copy(out_heavy, renamed_pdb)
        run(f"bash mutate_residues.sh {renamed_pdb}")

    else:
        # No protonation changes; just carry on
        renamed_pdb = out_heavy.with_name(out_heavy.stem + '_renamed.pdb')
        shutil.copy(out_heavy, renamed_pdb)

    assert renamed_pdb is not None, "Internal error: renamed_pdb not set"

    # 6. Rename CD ILE and HIS
    run(f'sed -i "s/CD  ILE/CD1 ILE/g" {renamed_pdb}')
    run(f'sed -i "s/HIS/HIE/g" {renamed_pdb}')
    run(f'sed -i "s/HSD/HID/g" {renamed_pdb}')
    run(f'sed -i "s/HSE/HIE/g" {renamed_pdb}')

    # 7. Add TER
    ter_pdb = renamed_pdb.with_name(renamed_pdb.stem + '_TER.pdb')
    #run(f'python {AABY_dir}/Scripts/add_ter.py {renamed_pdb} {ter_pdb}')
    run([sys.executable, "-m", "aaby.scripts.add_ter", str(renamed_pdb), str(ter_pdb)])
    assert ter_pdb.exists(), "add_ter failed"

    # Always remove END lines
    with open(str(ter_pdb), 'r') as f_in:
        ter_cleaned = [line for line in f_in if not line.startswith('END')]
    with open(str(ter_pdb), 'w') as f_out:
        f_out.writelines(ter_cleaned)

    # Prepare for COBY or direct to TLEAP
    last_pdb_before_coby = ter_pdb
    coby_input_pdb = Path("COBY_input.pdb")
    shutil.copy(last_pdb_before_coby, coby_input_pdb)

    # 8. Run COBY if and only if coby-yaml or coby-args are given
    coby_should_run = bool(args.coby_yaml or args.coby_args)
    if coby_should_run:
        apls = {'CHL': .41, 'DLPC': 0.61, 'DMPC': 0.60, 'DPPC': 0.62, 'DSPC': 0.60, 'DOPC': 0.67, 'POPC': 0.64, 'POPE': 0.56, 'DLPG': 0.66, 'DMPG': 0.65, 'DPPG': 0.68, 'DSPG': 0.67, 'DOPG': 0.71, 'POPG': 0.68, 'DOPS': 0.65, 'POPS': 0.62, 'POPA': 0.64, 'DAPC': 0.72, 'SDPC': 0.65, 'PSM': 0.58, 'SSM': 0.57}
        if args.coby_yaml:
            with open(args.coby_yaml) as f:
                coby_args = yaml.safe_load(f)
            mol_import = []
            ##NEW
            for idx, arg in enumerate(coby_args):
                if isinstance(arg, str):
                    if arg == "-membrane":
                        coby_args.insert(idx + 1, 'optimize_run:False')
                        coby_args.insert(idx + 1, 'grid_maker_grouping_algorithm:no_groups')
                    if arg.split(':')[0] == 'lipid':
                        mol_import.append(arg.split(':')[1])
                        coby_args[idx] = coby_args[idx] + f":apl:{apls[arg.split(':')[1]]}:params:Amber"
            for mol in mol_import:
                coby_args.append('-molecule_import')
                coby_args.append(f'file:{MODELS_DIR / (mol + ".pdb")}')
                coby_args.append(f'moleculetype:{mol}')
                coby_args.append('params:Amber')

            coby_args.append('-out_sys')
            coby_args.append('COBY.pdb')
            coby_args.append('-out_top')
            coby_args.append('COBY.top')

            with open('coby_input_params.yaml', 'w') as f:
                f.write(yaml.dump(coby_args))

            ## newparams ended

            if not isinstance(coby_args, list):
                print(f"YAML file must be a list of arguments, got {type(coby_args)}")
                sys.exit(1)
            coby_args = substitute_protein_pdb(coby_args, coby_input_pdb)
            coby_cmd = ["python3", "-m", "COBY"] + coby_args
        else:
            coby_args = substitute_protein_pdb(args.coby_args, coby_input_pdb)
            coby_cmd = ["python3", "-m", "COBY"] + coby_args
        run(coby_cmd)
        coby_pdb = Path('COBY.pdb')
        assert coby_pdb.exists(), "COBY membrane build failed"
        # 9. Add TER to COBY, then rename POPC and chains
        make_coby_ter_pdb('COBY.pdb', str(coby_input_pdb), 'COBY_TER.pdb')
        #run(f'python {AABY_dir}/Scripts/renamePOPC.py')
        run([sys.executable, "-m", "aaby.scripts.renamePOPC"])
        #run(f'python {AABY_dir}/Scripts/renameChain.py')
        run([sys.executable, "-m", "aaby.scripts.renameChain"])
        run(f'cp COBY_TER_lipid21_chains.pdb input4amber.pdb')
        # Always add TER after POPC/chain renaming if needed
        # You can re-use the existing add_ter.py for this purpose if needed
    else:
        # Not COBY: (If you want to split POPC for standalone systems, you could add it here)
        run(f'cp {coby_input_pdb} input4amber.pdb')
        pass

    # 10. Run tleap

    # 10.a Run antechamber
    if args.antechamber:
        antechamber(mol2=args.mol2, nc=args.nc, input_pdb=str(out_renamedligandatoms), chain=args.mol2chain)

    leap = auto_detect_types(water_model=args.water)
    run(f"sed -i '1s/^/{leap}/' tleap.in")
    run(f"sed -i '1s/^/{leap}/' tleap_solv.in")

    run('tleap -f tleap.in')
    prmtop, inpcrd, top_file, gro_file, topol = "system.prmtop","system.inpcrd", "system.top", "system.gro", "topol"
    #run(f'python {AABY_dir}/Scripts/convert_and_split.py {prmtop} {inpcrd} {top_file} {gro_file} {topol}')
    run([sys.executable, "-m", "aaby.scripts.convert_and_split", prmtop, inpcrd, top_file, gro_file, topol])
    run(f'gmx editconf -f input4amber.pdb -o input4amber.gro; am=$(tail -n 1 input4amber.gro); sm=$(tail -n 1 system.gro); sed -i "s/$sm/$am/" system.gro')


    # 11. Create topology that includes solvent
    ## instert water and ions insert-molecules
    run('gmx editconf -f input4amber.pdb -o input4amber_solvX.pdb')
    nr_molecules = 0
    nr_atoms = 0 #'OPC': 'water.opc', 'TIP3P': 'water.tip3p', 'TIP4PEW': 'water.tip4pew'
    nr_atoms_water = {'OPC': 4, 'TIP3P': 3, 'TIP4PEW': 4}
    #run(f'gmx insert-molecules -f input4amber_solvX.pdb -ci {AABY_dir}/models/{args.water.lower()}.gro -o input4amber_solv{nr_molecules}.pdb -nmol 1')
    water_model_path = MODELS_DIR / f"{args.water.lower()}.gro"
    run(
        f"gmx insert-molecules -f input4amber_solvX.pdb "
        f"-ci {water_model_path} "
        f"-o input4amber_solv{nr_molecules}.pdb -nmol 1"
    )
    nr_molecules += 1
    nr_atoms += nr_atoms_water[args.water]
    # for ion in args.ions.split(','):
    #     run(f'mkdir -p models')
    #     run(f'sed "s/XX /{ion}/g" {AABY_dir}/models/ion.pdb > models/tmp.pdb')
    #     run(f'gmx insert-molecules -f input4amber_solv{nr_molecules-1}.pdb -ci models/tmp.pdb -o input4amber_solv{nr_molecules}.pdb -nmol 1')
    #     nr_molecules += 1
    #     nr_atoms += 1
    for ion in args.ions.split(','):
        # run("mkdir -p models")
        ion_template = MODELS_DIR / "ion.pdb"
        # run(f'sed "s/XX /{ion}/g" {ion_template} > models/tmp.pdb')
        run(f'sed "s/XX /{ion}/g" {ion_template} > tmp.pdb')
        # run(
        #     f"gmx insert-molecules -f input4amber_solv{nr_molecules-1}.pdb "
        #     f"-ci models/tmp.pdb "
        #     f"-o input4amber_solv{nr_molecules}.pdb -nmol 1"
        # )
        run(
            f"gmx insert-molecules -f input4amber_solv{nr_molecules-1}.pdb "
            f"-ci tmp.pdb "
            f"-o input4amber_solv{nr_molecules}.pdb -nmol 1"
        )
        nr_molecules += 1
        nr_atoms += 1

    # with open(f"input4amber_solv{nr_molecules-1}.pdb", "rb") as f:
    #     after_insert = sum(1 for _ in f)

    # with open(f"input4amber_solvX.pdb", "rb") as f:
    #     before_insert = sum(1 for _ in f)

    difference_lines = nr_atoms + 2
    inserted_lines = []
    with open(f"input4amber_solv{nr_molecules-1}.pdb", "r") as f:
        lines = f.readlines()
        for line in lines[-difference_lines:]:
            inserted_lines.append(line)



    with open(f"input4amber.pdb", "r") as f:
        og_lines = f.readlines()
        original_length = len(og_lines)

    # print("OG LIENS", og_lines)

    with open('input4amber_solv.pdb','w') as f:
        for line in og_lines[:-2]:

            f.write(line)
        for line in inserted_lines:

            f.write(line)



    run('tleap -f tleap_solv.in')
    prmtop, inpcrd, top_file, gro_file, topol = "system_solv.prmtop","system_solv.inpcrd", "system_solv.top", "system_solv.gro", "topol_solv"
    #run(f'python {AABY_dir}/Scripts/convert_and_split.py {prmtop} {inpcrd} {top_file} {gro_file} {topol}')
    run([sys.executable, "-m", "aaby.scripts.convert_and_split", prmtop, inpcrd, top_file, gro_file, topol])

    run(f'grep include topol_solv.top > topol_nowater.top')
    run(f'grep -v include topol.top >> topol_nowater.top')

    # 12. Resolvate
    #run(f'python {AABY_dir}/Scripts/resolvate_replicas.py --base {base}_AABY --replicas {args.replicas} --water {args.water} --ions {args.ions} --conc {args.conc} --membrane {coby_should_run}')
    cmd = [
        sys.executable, "-m", "aaby.scripts.resolvate_replicas",
        "--base", f"{base}_AABY",
        "--replicas", str(args.replicas),
        "--water", args.water,
        "--ions", args.ions,
        "--conc", str(args.conc),
        "--mol2", args.mol2,
    ]

    if coby_should_run:
        cmd.append("--membrane")

    run(cmd)
    if args.replicas == 1:
        if args.mol2 is None:
            #run(f'python {AABY_dir}/Scripts/prepare_for_simulations.py {base}_AABY')
            run([sys.executable, "-m", "aaby.scripts.prepare_for_simulations",f"{base}_AABY"])
        else:
            #run(f'python {AABY_dir}/Scripts/prepare_for_simulations.py {base}_AABY {args.mol2}')
            run([sys.executable, "-m", "aaby.scripts.prepare_for_simulations", f"{base}_AABY", args.mol2])
        


    else:
        for rep in range(1, args.replicas + 1):
            os.chdir(f'r{rep}')
            if args.mol2 is None:
                #run(f'python {AABY_dir}/Scripts/prepare_for_simulations.py {base}_AABY')
                run([sys.executable, "-m", "aaby.scripts.prepare_for_simulations",f"{base}_AABY"])

            else:
                #run(f'python {AABY_dir}/Scripts/prepare_for_simulations.py {base}_AABY {args.mol2}')
                run([sys.executable, "-m", "aaby.scripts.prepare_for_simulations", f"{base}_AABY", args.mol2])


            os.chdir("..")

    if coby_should_run:
        src_mdps = MDPS_DIR / "MEMBRANE"
    else:
        src_mdps = MDPS_DIR / "SOLUTION"

    shutil.copytree(src_mdps, "mdps", dirs_exist_ok=True)

    total_time = time.time() - start_time
    print(f"\n✅ All done. Your system is ready for GROMACS.")
    print(f"Total build time: {total_time:.1f} seconds ({total_time/60:.2f} minutes)")

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import subprocess
import sys

def run(cmd, return_output=False):
    print("[RUN]", cmd)
    if return_output:
        result = subprocess.run(cmd, shell=True, capture_output=True, text=True)
        # result.stdout is the output, result.stderr is error output
        return result.stdout
    else:
        subprocess.run(cmd, shell=True, check=True)

def make_ndx(input_gro, mol2=None):
    run(f'echo q |  gmx make_ndx -f {input_gro}.gro -o tmp.ndx > /dev/null')
    groups = run(f'grep "\[" tmp.ndx  | wc -l', return_output=True)
    print(groups)
    groups = int(groups.strip()) - 1
    print(groups)
    if mol2 is None:
        run(f"printf '1 \n name {groups + 1} SOLU \n  r PA |  r PH- |  r AR |  r PS |  r ST |  r MY |  r PC |  r DHA |  r SA |  r SPM |  r LAL |  r PGR |  r OL |  r PE |  r CHL  \n name {groups + 2} MEMB \n {groups + 1} | {groups + 2} \n ! {groups + 3} \n name {groups + 4} SOLV \n q \n' | gmx make_ndx -f {input_gro}.gro -o index.ndx")
    else:
        run(f"printf '1 | \"{mol2}\" \n name {groups + 1} SOLU \n  r PA |  r PH- |  r AR |  r PS |  r ST |  r MY |  r PC |  r DHA |  r SA |  r SPM |  r LAL |  r PGR |  r OL |  r PE |  r CHL  \n name {groups + 2} MEMB \n {groups + 1} | {groups + 2} \n ! {groups + 3} \n name {groups + 4} SOLV \n q \n' | gmx make_ndx -f {input_gro}.gro -o index.ndx")


def main(base="system", mol2=None):
    input_gro = f"{base}"
    final_gro = f"{base}"
    # 1. Make index file
    make_ndx(input_gro, mol2)
    print(f"[DONE] See {final_gro} for output.")

if __name__ == "__main__":
    main(sys.argv[1], sys.argv[2] if len(sys.argv) > 2 else None)



import re
import sys
from pathlib import Path

def extract_residue_mapping(old_pdb, new_pdb):
    """Map (resname, chain, resid) in old to new resid."""
    def get_residues(pdb_path):
        residues = []
        with open(pdb_path) as f:
            for line in f:
                if line.startswith("ATOM") or line.startswith("HETATM"):
                    resname = line[17:20].strip()
                    chain = line[21]
                    resid = int(line[22:26])
                    atom = line[12:16].strip()
                    key = (resname, chain, atom)
                    residues.append((key, resid))
        return residues

    old = get_residues(old_pdb)
    new = get_residues(new_pdb)

    mapping = {}
    for (okey, old_resid), (_, new_resid) in zip(old, new):
        resname, chain, atom = okey
        mapping[(resname, chain, old_resid)] = new_resid
    return mapping

def renumber_ssbond(ssbond_txt, mapping, output_txt):
    with open(ssbond_txt) as f:
        lines = f.readlines()

    updated_lines = []
    for line in lines:
        match = re.match(r"(SSBOND\s+\d+\s+CYS\s+)(\w)\s+(\d+)(\s+CYS\s+)(\w)\s+(\d+)", line)
        if match:
            chain1, res1 = match.group(2), int(match.group(3))
            chain2, res2 = match.group(5), int(match.group(6))
            new_res1 = mapping.get(("CYS", chain1, res1), res1)
            new_res2 = mapping.get(("CYS", chain2, res2), res2)
            new_line = (f"{match.group(1)}{chain1:>2}{new_res1:>4}{match.group(4)}"
                        f"{chain2:>2}{new_res2:>4}{line[match.end():]}")
            updated_lines.append(new_line)
        else:
            updated_lines.append(line)

    with open(output_txt, "w") as f:
        f.writelines(updated_lines)

def parse_renumbered_ssbond(ssbond_renumbered_txt):
    # returns set of (chain, resid) for all CYX (formerly CYS) in ssbonds
    ss_cyx = set()
    with open(ssbond_renumbered_txt) as f:
        for line in f:
            if line.startswith('SSBOND'):
                chain1 = line[16].strip()
                res1 = int(line[17:21].strip())
                chain2 = line[30].strip()
                res2 = int(line[31:35].strip())
                ss_cyx.add((chain1, res1))
                ss_cyx.add((chain2, res2))
    return ss_cyx

def write_cyx_pdb(pdbfile, cyx_set, outname):
    with open(pdbfile) as f:
        lines = f.readlines()
    with open(outname, 'w') as f:
        for line in lines:
            if (line.startswith("ATOM") or line.startswith("HETATM")):
                resname = line[17:20]
                chain = line[21]
                resid = int(line[22:26])
                # print(cyx_set)
                if resname.strip() == "CYS" and (chain, resid) in cyx_set:
                    line = line[:17] + "CYX" + line[20:]
            f.write(line)

# Usage
if len(sys.argv) < 2:
    print("Usage: python re_ss.py <pdb_file>")
    sys.exit(1)

pdb_file = sys.argv[1]
pdb_file_re = pdb_file.replace(".pdb", "_re.pdb")

mapping = extract_residue_mapping(
    pdb_file,
    pdb_file_re
)

renumber_ssbond("SSBOND.txt", mapping, "SSBOND_renumbered.txt")

with open('SSBOND_renumbered.txt') as f:
    lines = f.readlines()

with open('tleap_SSBONDs.txt', 'w') as f:
    for line in lines:
        if not line.startswith("SSBOND"):
            continue
        res1 = line[17:21].strip()
        res2 = line[31:35].strip()
        f.write(f'bond ramp.{res1}.SG ramp.{res2}.SG\n')

#def insert_ssbonds_into_tleap(tleap_in_path='Scripts/tleap.in', ssbond_path='tleap_SSBONDs.txt'):
def insert_ssbonds_into_tleap(tleap_in_path='tleap.in', ssbond_path='tleap_SSBONDs.txt'):
    tleap_in = Path(tleap_in_path)
    ssbonds = Path(ssbond_path)
    if not tleap_in.exists() or not ssbonds.exists():
        print(f"Warning: {tleap_in} or {ssbonds} does not exist. Skipping SSBOND insertion.")
        return

    with tleap_in.open() as f:
        lines = f.readlines()

    # Find the line with loadpdb (should be only one)
    for idx, line in enumerate(lines):
        if 'loadpdb' in line:
            loadpdb_idx = idx
            break
    else:
        print("Could not find 'loadpdb' in tleap.in!")
        return

    with ssbonds.open() as f:
        ssbond_lines = f.readlines()

    # Insert after loadpdb line
    new_lines = lines[:loadpdb_idx + 1] + ssbond_lines + lines[loadpdb_idx + 1:]

    # Overwrite tleap.in
    with tleap_in.open('w') as f:
        f.writelines(new_lines)

    print(f"Inserted {ssbonds} after loadpdb in {tleap_in}")

# --- New Part: Write *_re_ssbond.pdb with CYX ---

cyx_set = parse_renumbered_ssbond("SSBOND_renumbered.txt")
pdb_file_re_ssbond = Path(pdb_file).with_name(Path(pdb_file_re).stem + "_ssbond.pdb")
write_cyx_pdb(pdb_file_re, cyx_set, pdb_file_re_ssbond)
print(f"Wrote {pdb_file_re_ssbond}")
